Attribute VB_Name = "basShell"
Option Explicit

Public Const conNoError = 0&
Public Const conMaxPath = 260
Public Const conErrorExtendedError = 1208&

' Messages sent to callback function.
Private Const BFFM_INITIALIZED = 1
Private Const BFFM_SELCHANGED = 2

' Messages to browser from callback function.
Private Const WM_USER = &H400
Private Const BFFM_SETSTATUSTEXT = (WM_USER + 100)
Private Const BFFM_ENABLEOK = (WM_USER + 101)
Private Const BFFM_SETSELECTION = (WM_USER + 102)

Private Type BROWSEINFO
    hwndOwner As Long           ' Owner
    pidlRoot As Long            ' Can be null
    strDisplayName As String    ' Recieves display name of folder
    strTitle As String          ' title/instructions for user
    ulFlags As Long             ' 0 or BIF constants
    lpfn As Long                ' Address for callback
    lParam As Long              ' Passes to callback
    iImage As Long              ' index to the system image list
End Type

' Shell32 APIs
Public Declare Function SHGetSpecialFolderLocation Lib "Shell32.DLL" (ByVal hwndOwner As Long, ByVal Folder As Long, ByRef idl As Long) As Long
Private Declare Function SHGetPathFromIDList Lib "Shell32.DLL" (ByVal idl As Long, ByVal Path As String) As Long
Private Declare Function SHBrowseForFolder Lib "Shell32.DLL" (ByRef bi As BROWSEINFO) As Long

Private Declare Function SendMessage Lib "user32" Alias "SendMessageA" (ByVal hWnd As Long, ByVal wMsg As Long, ByVal wParam As Long, ByVal lParam As Any) As Long
Private Declare Function GlobalFree Lib "kernel32" (ByVal hMem As Long) As Long

' Shell constants
Public Const BIF_RETURNONLYFSDIRS = &H1&
Public Const BIF_DONTGOBELOWDOMAIN = &H2&
Public Const BIF_STATUSTEXT = &H4&
Public Const BIF_RETURNFSANCESTORS = &H8&
Public Const BIF_BROWSEFORCOMPUTER = &H1000&
Public Const BIF_BROWSEFORPRINTER = &H2000&

Public Const CSIDL_DESKTOP = &H0
Public Const CSIDL_PROGRAMS = &H2
Public Const CSIDL_CONTROLS = &H3
Public Const CSIDL_PRINTERS = &H4
Public Const CSIDL_PERSONAL = &H5
Public Const CSIDL_FAVORITES = &H6
Public Const CSIDL_STARTUP = &H7
Public Const CSIDL_RECENT = &H8
Public Const CSIDL_SENDTO = &H9
Public Const CSIDL_BITBUCKET = &HA
Public Const CSIDL_STARTMENU = &HB
Public Const CSIDL_DESKTOPDIRECTORY = &H10
Public Const CSIDL_DRIVES = &H11
Public Const CSIDL_NETWORK = &H12
Public Const CSIDL_NETHOOD = &H13
Public Const CSIDL_FONTS = &H14
Public Const CSIDL_TEMPLATES = &H15
Public Const CSIDL_COMMON_STARTMENU = &H16
Public Const CSIDL_COMMON_PROGRAMS = &H17
Public Const CSIDL_COMMON_STARTUP = &H18
Public Const CSIDL_COMMON_DESKTOPDIRECTORY = &H19
Public Const CSIDL_APPDATA = &H1A
Public Const CSIDL_PRINTHOOD = &H1B

Public Function BrowseForFolder(ByVal lngCSIdl As Long, ByVal lngBifFlags As Long, ByRef strFolder As String, Optional ByVal strTitle As String = "Select Folder", Optional ByVal hWnd As Long = 0) As Long
    ' Browse for a folder
    
    ' In:
    '   lngCSIDL:
    '       The CSIDL_* constant that specifies a start directory
    '   lngBifFlags:
    '       A BIF_* bit mask that limits item selections
    '   strFolder:
    '       Buffer in which to place the folder that is retrieved
    '   hWnd: (optional)
    '       Window handle for the dialog's owner
    '   strTitle: (optional)
    '       Title for the dialog
    '
    ' Out:
    '   Return Value:
    '       conNoError on success, conErrorExtendedError on failure
    ' Note:
    '
 
    Dim bi As BROWSEINFO
    Dim lngReturn As Long
    Dim lngIDL As Long
    
    ' First get the special folder that lngCSIDL and lngID specify
    If SHGetSpecialFolderLocation(hWnd, lngCSIdl, lngIDL) = 0 Then
    
        ' Set up the browse structure
        With bi
            .hwndOwner = hWnd
            .pidlRoot = lngIDL
            .strDisplayName = Space(conMaxPath)
            .strTitle = strTitle
            .ulFlags = lngBifFlags
            'If frmTestBrowser.UseCallback Then
                '.lpfn = FnPtrToLong(AddressOf CallBackFunction)
            'End If
        End With

        ' Open the browse dialog
        lngIDL = SHBrowseForFolder(bi)
        
        ' If the dialog succeeded
        If lngIDL <> 0 Then
            strFolder = Space(conMaxPath)
            
            ' Try to resolve the lngIDL into a real path
            If CBool(SHGetPathFromIDList(lngIDL, strFolder)) Then
                ' Trim the null characters
                strFolder = TrimNull(strFolder)
                lngReturn = conNoError
            Else
                ' No real directory path is available, so at least return the virtual selection
                strFolder = TrimNull(bi.strDisplayName)
                lngReturn = conNoError
            End If
        Else
            ' Some unknown error has occurred
            lngReturn = conErrorExtendedError
        End If
        ' It's up to the caller to free the memory used by
        ' the ID list.
        Call GlobalFree(lngIDL)
        
    Else
        ' Some unknown error has occurred
        lngReturn = conErrorExtendedError
    End If
    
    BrowseForFolder = lngReturn
End Function

Private Function TrimNull(ByVal strValue As String) As String
    ' Find the first vbNullChar in a string, and return
    ' everything prior to that character. Extremely
    ' useful when combined with the Windows API function calls.
    
    ' In:
    '   strValue:
    '       Input text, possibly containing a null character
    '       (chr$(0), or vbNullChar)
    ' Out:
    '   Return Value:
    '       strValue trimmed on the right, at the location
    '       of the null character, if there was one.
    
    Dim intPos As Integer
    
    intPos = InStr(strValue, vbNullChar)
    Select Case intPos
        ' It's best to put the most likely case first.
        Case Is > 1
            ' Found in the string, so return the portion
            ' up to the null character.
            TrimNull = Left$(strValue, intPos - 1)
        Case 0
            ' Not found at all, so just
            ' return the original value.
            TrimNull = strValue
        Case 1
            ' Found at the first position, so return
            ' an empty string.
            TrimNull = ""
    End Select
End Function

Private Function FnPtrToLong(ByVal lngFnPtr As Long) As Long
    ' Given a function pointer as a Long, return a Long.
    ' Sure looks like this function isn't doing anything,
    ' and in reality, it's not.
    FnPtrToLong = lngFnPtr
End Function

Private Function CallBackFunction(ByVal hWnd As Long, ByVal lngMsg As Long, _
 ByVal lngParam As Long, ByVal lngData As Long) As Long
'    ' This is an example callback function, simply demonstrating
'    ' the kinds of things this function can do.
'
'    ' The formal declaration of this function is specified by the
'    ' SHBrowseForFolder API call, and cannot be changed.
'
'    Dim strFolder As String
'    Dim lngIDL As Long
'
'    Select Case lngMsg
'        Case BFFM_INITIALIZED
'            Call EnableOK(hWnd, False)
'            Call SetSelection(hWnd, frmTestBrowser.Selection)
'
'        Case BFFM_SELCHANGED
'            strFolder = Space(conMaxPath)
'
'            ' Try to resolve the lngIDL into a real path
'            If CBool(SHGetPathFromIDList(lngParam, strFolder)) Then
'                ' Trim the null characters
'                strFolder = TrimNull(strFolder)
'                Call SetStatus(hWnd, strFolder)
'                Call EnableOK(hWnd, True)
'            Else
'                ' Not a file system path, so
'                ' clear that status, and disable
'                ' the OK button.
'                Call SetStatus(hWnd, vbNullString)
'                Call EnableOK(hWnd, False)
'            End If
'    End Select
'    CallBackFunction = 0
End Function

Private Sub SetStatus(hWnd As Long, strText As String)
    ' Set the status text on the Shell browser dialog.
    Call SendMessage(hWnd, BFFM_SETSTATUSTEXT, 0, strText)
End Sub

Private Sub EnableOK(hWnd As Long, fEnable As Boolean)
    ' Enable the OK button on the Shell browser dialog.
    Dim lngEnable As Long
    
    If fEnable Then
        lngEnable = 1
    Else
        lngEnable = 0
    End If
    Call SendMessage(hWnd, BFFM_ENABLEOK, 0, lngEnable)
End Sub

Private Function SetSelection(hWnd As Long, varSel As Variant) As Long
    ' Set the selection in the Shell browser dialog. If
    ' varSel is numeric, the code assumes that the value is a
    ' PDIL, and calls SendMessage with that information. If
    ' not, the code converts the value to a string, and
    ' tells the browser that it's sending a string value instead.
    If IsNumeric(varSel) Then
        Call SendMessage(hWnd, BFFM_SETSELECTION, 0, CLng(varSel))
    Else
        Call SendMessage(hWnd, BFFM_SETSELECTION, 1, CStr(varSel & ""))
    End If
End Function

